package jp.co.atmarkit.aws.s3.action {
import com.adobe.webapis.awss3.AWSS3;
import com.adobe.webapis.awss3.AWSS3Event;
import com.adobe.webapis.awss3.Bucket;

import flash.data.EncryptedLocalStore;
import flash.events.IOErrorEvent;
import flash.events.MouseEvent;
import flash.utils.ByteArray;

import jp.co.atmarkit.aws.s3.core.Action;
import jp.co.atmarkit.aws.s3.core.CloseWithDataEvent;
import jp.co.atmarkit.aws.s3.helper.MainHelper;

import mx.controls.Alert;
import mx.controls.dataGridClasses.DataGridColumn;
import mx.events.CloseEvent;
import mx.events.FlexEvent;
import mx.events.MenuEvent;
import mx.formatters.DateFormatter;
import mx.formatters.NumberFormatter;
import mx.logging.ILogger;
import mx.logging.Log;
import mx.managers.CursorManager;
import mx.utils.ObjectUtil;


/**
 * MainActionクラス。イベントのハンドリングを行います。
 */
public class MainAction extends Action {

//--------------------------------------
//  Component
//--------------------------------------
    
    /** Helperコンポーネントです。 */
    public var helper:MainHelper;
    
	/** ログコンポーネントです。 */
    private var logger:ILogger = Log.getLogger("MainAction"); 
	    
    /** 日付フォーマッタ */
    private var dateFormatter:DateFormatter;

    /** 数値フォーマッタ */
    private var numberFormatter:NumberFormatter;

//--------------------------------------
//  AWS S3 Component
//--------------------------------------
    
    /** AWS S3用ユーティリティコンポーネントです。 */
	private var s3:AWSS3;
	
	/** アクセスキーです。 */
	private var accessKey:String = "";
	
	/** シークレットアクセスキーです。 */
	private var secretAccessKey:String = "";

//--------------------------------------
//  Constructor
//--------------------------------------

	/**
	 * MainAction オブジェクトを作成します。
	 */
    public function MainAction() {
        super();        

        dateFormatter = new DateFormatter();
        dateFormatter.formatString = "YYYY/MM/DD HH:NN";
        
        numberFormatter = new NumberFormatter();
        numberFormatter.useThousandsSeparator = true;
    }
    
//--------------------------------------
//  Init
//--------------------------------------

    /**
     * 初期化処理
     * 
     * @param event FlexEvent
     */
    protected override function onCreationCompleteHandler(event:FlexEvent):void {
	    logger.debug(">>onCreationCompleteHandler");
        
        super.onCreationCompleteHandler(event);
        
        initComponents();
        
        initAwsS3();
        
        helper.initListBucketsButtonHandler(listBucketsButtonClickHandler);
        helper.initMenuBarHandler(menuBarItemClickHandler);
        helper.initBucketDataGridDoubleClickHandler(bucketDataGridDoubleClickHandler);
    }
    
    /**
     * コンポーネントを初期化します。
     */
    private function initComponents():void {
        // コンポーネントを手動でDIします。
        helper = new MainHelper();
        helper.initView(super.document);
    }
    
    /**
     * AWS S3 関連オブジェクトを初期化します。
     */
    private function initAwsS3():void {
		// AWSキーの初期化
		var accessKeyBytes:ByteArray;
		var secretAccessKeyBytes:ByteArray;
		
		try {
			accessKeyBytes = EncryptedLocalStore.getItem("accessKey");
			secretAccessKeyBytes = EncryptedLocalStore.getItem("secretAccessKey");
		} catch (e:Error) {
			EncryptedLocalStore.reset();
		}

		if (accessKeyBytes != null && secretAccessKeyBytes != null) {
			accessKey = accessKeyBytes.readUTFBytes(accessKeyBytes.length);
			secretAccessKey = secretAccessKeyBytes.readUTFBytes(secretAccessKeyBytes.length);
			
			helper.enableListBucketsButton(true);
		} else {
		    helper.enableListBucketsButton(false);
		}
    }
    
//--------------------------------------
//  Event Handler
//--------------------------------------
    
    /**
     * メニューバーのアイテムクリックハンドラです。
     * 
     * @param event MenuEvent オブジェクトです。
     */
    private function menuBarItemClickHandler(event:MenuEvent):void {
	    logger.debug(">>menuBarItemClickHandler");
	    
	    if(event.label == "キーの登録") {
	        helper.popUpKeyRegisterView(keyRegisterViewCloseHanlder);
	    } else if(event.label == "終了") {
	        helper.showCloseApplicationAlert(alertCloseHandler);
	    } else if(event.label == "バージョン情報") {
	        
	    }
    }
    
    /**
     * バケット一覧のダブルクリックハンドラです。
     * 
     * @param event MouseEvent オブジェクトです。
     */
    private function bucketDataGridDoubleClickHandler(event:MouseEvent):void {
	    logger.debug(">>bucketDataGridDoubleClickHandler");
	    
	    var storageItem:Object = helper.getBucketDataGridSelectedItem();
	    
        if(storageItem is Bucket) {
            var bucket:Bucket = Bucket(storageItem);
            s3.listObjects(bucket.name);
            
            helper.setBucketName(bucket.name);

		    setBusyCursor();
        }
    }
    
    /**
     * 画面を閉じるアラートの閉じるハンドラです。
     * 
     * @param event CloseEvent オブジェクトです。
     */
    private function alertCloseHandler(event:CloseEvent):void {
	    logger.debug(">>alertCloseHandler");
        if(event.detail == Alert.OK) {
            helper.exitApplication();
        }
    }

//--------------------------------------
//  AWS S3 Event Handler
//--------------------------------------

    /**
     * ListBucketsボタンのクリックハンドラです。
     * 
     * @param event MouseEvent オブジェクトです。
     */
    private function listBucketsButtonClickHandler(event:MouseEvent):void {
	    logger.debug(">>listBucketsButtonClickHandler");
        
        if(s3 == null) {
    		s3 = new AWSS3(this.accessKey, this.secretAccessKey);
    		
    		s3.addEventListener(IOErrorEvent.IO_ERROR, awsS3IOErrorHandler);
    		s3.addEventListener(AWSS3Event.ERROR, awsS3ErrorHanlder);
    		s3.addEventListener(AWSS3Event.LIST_BUCKETS, awsS3ListBucketsHandler);
    		s3.addEventListener(AWSS3Event.LIST_OBJECTS, awsS3ListObjectsHandler);
    //		s3.addEventListener(AWSS3Event.BUCKET_CREATED, onBucketCreated);
    //		s3.addEventListener(AWSS3Event.BUCKET_DELETED, onBucketDeleted);
    //		s3.addEventListener(AWSS3Event.OBJECT_DELETED, onObjectDeleted);
    //		s3.addEventListener(AWSS3Event.OBJECT_SAVED, onObjectSaved);
    //		s3.addEventListener(AWSS3Event.OBJECT_RETRIEVED, onFileDownloaded);
    //		s3.addEventListener(AWSS3Event.REQUEST_FORBIDDEN, onRequestForbidden);
        }

		setBusyCursor();

        helper.setBucketName("");

        s3.listBuckets();
    }
    
    /**
     * ListBuckets処理の結果ハンドラです。
     * 
     * @param event AWSS3Event オブジェクトです。
     */
    private function awsS3ListBucketsHandler(event:AWSS3Event):void {
	    logger.debug(">>awsS3ListBucketsHandler");
	    
	    removeBusyCursor();
	    
	    helper.changeStorageItemViewStackSelectedIndex(0);

	    helper.setBucketList(event.data);
    }
    
    /**
     * ListObjects処理の結果ハンドラです。
     * 
     * @param event AWSS3Event オブジェクトです。
     */
    private function awsS3ListObjectsHandler(event:AWSS3Event):void {
	    logger.debug(">>awsS3ListObjectsHandler");
	    
	    removeBusyCursor();
	    
	    helper.changeStorageItemViewStackSelectedIndex(1);
	    
	    helper.setS3ObjectList(event.data);
    }

    /**
     * AWSS3のIOErrorハンドラです。
     * 
     * @param event AWSS3Event オブジェクトです。
     */
    private function awsS3IOErrorHandler(event:IOErrorEvent):void {
	    logger.debug(">>awsS3IOErrorHandler");
        
	    Alert.show(ObjectUtil.toString(event));
    }
    
    /**
     * AWSS3のErrorハンドラです。
     * 
     * @param event AWSS3Event オブジェクトです。
     */
    private function awsS3ErrorHanlder(event:AWSS3Event):void {
	    logger.debug(">>awsS3ErrorHanlder");
        
	    Alert.show(ObjectUtil.toString(event));
    }

//--------------------------------------
//  KeyRegisterView
//--------------------------------------

    /**
     * キー登録画面の閉じるイベントハンドラです。
     * 
     * @param event CloseEvent オブジェクトです。
     */
    private function keyRegisterViewCloseHanlder(event:CloseEvent):void {
	    logger.debug(">>keyRegisterViewCloseHanlder");
        
        if(event is CloseWithDataEvent && CloseWithDataEvent(event).data != null) {
    		var accessKeyBytes:ByteArray = new ByteArray();
    		var secretAccessKeyBytes:ByteArray = new ByteArray();
    
            accessKey = CloseWithDataEvent(event).data.accessKey as String;
            secretAccessKey = CloseWithDataEvent(event).data.secretAccessKey as String;
            
    		accessKeyBytes.writeUTFBytes(accessKey);
    		secretAccessKeyBytes.writeUTFBytes(secretAccessKey);
    		
    		EncryptedLocalStore.setItem("accessKey", accessKeyBytes);
    		EncryptedLocalStore.setItem("secretAccessKey", secretAccessKeyBytes);
            
            helper.enableListBucketsButton(true);
        }
    }

//--------------------------------------
//  Utility
//--------------------------------------
    
    /**
     * Bucket.creationDate用のラベルファンクション。
     * 
     * @param item 行に割り当てられたオブジェクトです。
     * @param column ラベルファンクションが割り当てられた列です。
     */
    public function setCreationDate(item:Object,column:DataGridColumn):String {
        return dateFormatter.format(item.creationDate);
    }
    
    /**
     * S3Object.lastModified用のラベルファンクション。
     * 
     * @param item 行に割り当てられたオブジェクトです。
     * @param column ラベルファンクションが割り当てられた列です。
     */
    public function setLastModified(item:Object,column:DataGridColumn):String {
        return dateFormatter.format(item.lastModified);
    }

    /**
     * S3Object.size用のラベルファンクション。
     * 
     * @param item 行に割り当てられたオブジェクトです。
     * @param column ラベルファンクションが割り当てられた列です。
     */
    public function setSize(item:Object,column:DataGridColumn):String {
        return numberFormatter.format(item.size);
    }

    /**
     * ビジーカーソルを表示します。
     */
	private function setBusyCursor():void {
		CursorManager.setBusyCursor();
	}

    /**
     * ビジーカーソルを削除します。
     */
	private function removeBusyCursor():void {
		CursorManager.removeBusyCursor();
	}
}
}